{
  description = "The Flake";

  inputs = {
    nixpkgs-unstable.url = "github:nixos/nixpkgs/nixos-unstable";
    nixpkgs.url = "github:nixos/nixpkgs/nixos-24.11";
    home-manager = {
      url = "github:nix-community/home-manager/release-24.11";
      inputs.nixpkgs.follows = "nixpkgs";
    };
    nix-bitcoin = {
      url = "github:fort-nix/nix-bitcoin/release";
    };
    nix-comfyui.url = "github:dyscorv/nix-comfyui";
  };

  outputs = { nixpkgs,
              nixpkgs-unstable,
              home-manager,
              nix-bitcoin,
              nix-comfyui,
              ... }:
    let
      system = "x86_64-linux";
      username = "chris";
      pkgsForSystem = import nixpkgs {
        inherit system;
        config = { allowUnfree = true; };
      };

      lib = nixpkgs.lib;
      unstable = import nixpkgs-unstable {
        inherit system;
        config = { allowUnfree = true; };
        overlays = [
          (import ./python312overlay.nix)
          nix-comfyui.overlays.default
        ];
      };

    in  {
      nixosConfigurations = {
        shen = lib.nixosSystem {
          inherit system;
          pkgs = pkgsForSystem;
          specialArgs = { inherit unstable; };
          modules = [
            ./system/shen/configuration.nix
            home-manager.nixosModules.home-manager
            {
              home-manager.useGlobalPkgs = true;
              home-manager.useUserPackages = true;
              home-manager.users.chris = import ./chris/home.nix;
            }
          ];
        };
        hoid = lib.nixosSystem {
          inherit system;
          pkgs = pkgsForSystem;
          specialArgs = { inherit unstable; };
          modules = [
            ./system/hoid/configuration.nix
            home-manager.nixosModules.home-manager
            {
              home-manager.useGlobalPkgs = true;
              home-manager.useUserPackages = true;
              home-manager.users.chris = import ./chris/home.nix;
            }
          ];
        };
      };
    };
}
